#include "i2c.h"
#include "vxp202.h"
#include <stdio.h>
#include <time.h>

static float I2C_DLY;
static int I2C_PORT;
static byte i2c_ndev=0;
static byte i2c_device[256];

/* delay for dly ns */
void I2CWait(unsigned int dly) {
  unsigned long i,n;

  n=(unsigned long)(I2C_DLY*(float)dly)+1;
  for (i=0;i<n;i++);
}

int I2CGetSDA() {
  return I2C_PORT & VxP_I2C_DATA;
}

int I2CGetSCL() {
  return I2C_PORT & VxP_I2C_CLK;
}

void I2CSetSDA(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_DATA;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_CLK;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    #ifdef DEBUG
      I2CProtocol();
    #endif
  #endif
}

void I2CSetSCL(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_CLK;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_DATA;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    #ifdef DEBUG
      I2CProtocol();
    #endif
  #endif
}

void I2CStart() {
  /*    ____
            |_____
        __________
                    CLCK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_HDSTA);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  /* now ready to transmit data */
}

void I2CStop() {
  /*
          __
     ____|
     _______
             CLCK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUSTO);
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_BUF);
  /* transmission completed */
}

void I2CHigh() {
  /*
      _______ H
              L
         _
      __| |__ CLK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

void I2CLow() {
  /*
              H
      _______ L
         _
      __| |__ CLK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

int I2CAck() {
  int ack,value;
  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read ACK / data
  */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE);
  ack=VxP_PortRead(VxP_I2C_CTRL);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  return !(ack & VxP_I2C_DATA_BACK);
}

int I2CSendByte(int data) {
  int i;

  for (i=0;i<8;i++) {
    if (data & (1 << i))
      I2CHigh();
    else
      I2CLow();
  }
  i=I2CAck();
  if (!i)
    return 0;
  return 1;
}

int I2CReadByte(void) {
  int i;
  int data=0;

  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read data
  */
  /* set data output bit to H, CLK L */
  return data;
}

void I2CInit() {
  clock_t t1,t2,dt;
  unsigned long i,cnt;

  #ifdef VxP202_
    VxP_PortWrite(VxP_I2C_CTRL,0);
    I2C_PORT = VxP_I2C_CLK | VxP_I2C_DATA | VxP_I2C_FRAME;
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    I2CWait(2*I2C_DLY_BUF);
  #endif

  cnt=1;
  do {
    cnt=cnt*2;
    t1=clock();
    for (i=0;i<cnt;i++);
    t2=clock();
    dt=t2-t1;
  } while (dt<20);
  #ifdef DOS
    I2C_DLY=((float)cnt/1e9)*((float)CLK_TCK/(float)dt);
  #else
  #endif
}

void I2CScanDevices(void) {
  int i;

  i2c_ndev=0;
  for (i=0; i<256; i++) {
printf(".");
    I2CStart();
    if (I2CSendByte(i)) {
      i2c_device[i2c_ndev]=i;
      i2c_ndev++;
    }
    I2CStop();
  }
}

int I2CNDev() {
  return i2c_ndev;
}

int I2CDevice(int i) {
  if (i2c_ndev==0)
    return -1;
  if ((0<=i) && (i<i2c_ndev))
    return i2c_device[i];
  return -1;
}

/* send size bytes to device dev */
int I2CSend(int ldev,byte *buffer,int size) {
  int dev;
  int i;
  int status;
  int fail;
  int maxfail;

  maxfail=5;
  dev=I2CDevice(ldev);
  if (dev<0)
    return -1;
  i=0;
  I2CStart();
  status=I2CSendByte(dev);
  do {
    fail=0;
    do {
      status=I2CSendByte(buffer[i]);
      fail++;
    } while ((!status) && (fail<maxfail));
    i++;
  } while ((i<size) && (fail<maxfail));
}



/* receive <=size bytes from dev */
int I2CRecv(int ldev,byte *buffer,int size) {
  int dev;

  dev=I2CDevice(ldev);
  if (dev<0)
    return -1;
}

